/*******************************************************************************

    TINNY ZENER TESTER

    C/ PIC12F675 E LCD 16x2 (MTODO LPLEX - 3 FIOS)


    AUTOR: CLAUDIO LARIOS

    DATA:21/04/2023

    Clock: 4MHz (INTERNO)

    Use CCS C compiler 3.2

    O BYTE DE CALIBRAO DEVER SER O ORIGINAL PARA FUNCIONAMENTO CORRETO.

    USO DIDTICO
    ESTE ARQUIVO  PARTE INTEGRANTE DO BLOG PICSOURCE

*******************************************************************************/
#include <12F675.h>//pic usado
#device adc=10
#use delay(clock=4000000)//freq. de clock
#fuses INTRC,NOWDT, NOCPD, NOPROTECT, MCLR, NOPUT, NOBROWNOUT//configurao INTRC_IO
#zero_ram//zera no nicio
//#rom  0x3ff = {0x345C} //coloca valor aproximado do oscall do chip




//bytes
#byte  trisio    = 0x85
#byte  gpio      = 0x05
#byte  t1con     = 0x10
#byte tmr0       = 0x01


#byte cmcon      = 0x19
#byte timer0     = 0x01
#byte option_reg = 0x81
#byte t1con      = 0x10
#byte adcon0     = 0x1f
#byte ansel      = 0x9f
#byte adresh     = 0x1e
#byte adresl     = 0x9e


//pinos do lcd (metodo Lariosplex - 3 fios)

#bit DB76  = 0x5.5  //pino 2 db7=direto db6=resistor 10k +cap 10nF ao gnd
#bit DB54  = 0x5.1  //pino 6 db5=direto db4=resistor 10k +cap 10nF ao gnd
#bit E_RS  = 0x5.2  //pino 5 E=direto RS=resistor 10k +cap 10nF ao gnd
//

//bit
//#bit  escm    = 0x5.5  //escala alta  pino 2 ('0') - 1k
//#bit  escl    = 0x5.2  //escala baixa pino 5 ('0') - 100 ohms

//#bit  desc    = 0x5.4  //descarga do capacitor cx - PINO 3 GP4 ('1')
//#bit  sw      = 0x5.3  //boto zerar

#bit  c2      = 0x1f.7 //sada do comparador 1
#bit  c1      = 0x1f.6 //sada do comparador 2
#bit  cmif    = 0x0c.6 //flag interrupo dos comparadores analgicos
#bit  t0if    = 0x0b.2 //flag de interrupo do timer 0
#bit  t1if    = 0x0C.0 //flag do timer1
#bit go_done  = 0x1f.1 //flag de nicio de converso adc


 //variveis globais


 int16 res_esr=0;

 int8 mil;
 int8 cent;
 int8 dez;
 int8 un;
 int1 RS;


 //DEFINE DO LCD LARIOSPLEX
#define vcarga 500 // 400 por tentativa e erro (entre 200 a 500)
#define vdescarga 500//400 idem
#define v_delay_clk 2  //2  // de 2 a 50 conforme LCD



/******************************************************************************

******************************************************************************/

/*******************************************************************************
                          SUBROTINAS
*******************************************************************************/

void delay_4ms(){delay_ms(4);}//delay 4ms
void descarregar(){ DB76=0;DB54=0;E_RS=0; delay_us(vdescarga);}//descarga do cap
void t_carga(){delay_us(vcarga);}//tempo para carga do cap
void clk(){ E_RS=1; delay_cycles(v_delay_clk); E_RS=0; }//gera clock par Lcd


/*******************************************************************************
                          SUBROTINAS de LCD
*******************************************************************************/


/*******************************************************************************
                         ENVIA NIBBLE
*******************************************************************************/
void env_nible(unsigned int8 nib){
    disable_interrupts(global);//necessrio p/ no atrapalhar tempos do lplex
    descarregar();//descarrega capacitores
    nib&=0x0f;//limita
    if(nib&0x01)DB54=1;//testa bit 0
    if(nib&0x04)DB76=1;//testa bit 2
    if(RS)E_RS=1;else E_RS=0;//comando ou dado
    t_carga();//tempo para carregar capacitor
    if(nib&0x02)DB54=1; else DB54=0;//testa bit 1
    if(nib&0x08)DB76=1; else DB76=0;//testa bit 3
    clk();//clock

    DB54=0; DB76=0; E_RS=0; //TESTE XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

    enable_interrupts(global);//religa interrupes

}

/*******************************************************************************
                          ENVIA DADOS PARA O LCD
*******************************************************************************/

void env_lcd_d(unsigned int8 dado){

 RS=1;//indica que enviar 'dados'
 env_nible(dado>>4);//high nibble primeiro
 env_nible(dado);//envia low nibble
 delay_4ms();//delay
}

/*******************************************************************************
                         ENVIA COMANDOS PARA LCD
*******************************************************************************/
void env_lcd_c(unsigned int8 comando){

 RS=0;//indica que enviar 'comandos'
 env_nible(comando>>4);//high nibble primeiro
 env_nible(comando);//low nibble
 delay_4ms();//delay
}

/*******************************************************************************
                         INICIALIZAO DO LCD
*******************************************************************************/

void Lcd_Init(){
  unsigned int8 a;

   descarregar();//descarrega capacitores
   RS=0; //enviar comando
   delay_ms(100);//delay inicial


   //xxxxxxxxxxxxxxxx
   //reset do lcd
   E_RS=0;
   DB54=1; DB76=1;
   t_carga();
   clk();
   clk();
   clk();
   descarregar();
   clk();
   clk();
   clk();
   delay_ms(100);//delay inicial
   //xxxxxxxxxxxxxxx


   for(a=0;a<3;a++) { env_nible(3);delay_4ms();}//envia 3x 0x03
   env_nible(2);//envia 0x02
   delay_4ms();//delay 4 ms
   env_lcd_c(0x28);//ajusta para 4 bits o modo de envio
   env_lcd_c(0x06);// no shift
   env_lcd_c(0x0c);//Display ON, Cursor OFF, Blink OFF
   env_lcd_c(0x01);//limpa tela
}

void delay500us(){
  delay_us(500);
}
void delay1seg(){
  delay_ms(1000);
}


void leitura_adc(){ //leitura ESR escala de 1.0 a 9.9 ohms
int16 soma=0;
int8 a;


  for(a=0;a<32;a++){
   go_done=1;//liga converso no pino 7 (an7)

   while(go_done);
   res_esr= (int16) adresh<<8;
   res_esr= res_esr  + adresl;
   soma= soma + res_esr;
  }
  res_esr= soma/32;
}


//==============================================================================
//         ROTINA MAIN
//==============================================================================

void main() {

   option_reg=0x8F; //timer0 /2 no pullup /watchdog 2,3 seg
   trisio=0b001001;//entradas (1) e saidas (0) / gp0 adc/ gp3 sw
   //ansel=0b00010001;//fosc/8 / canal 0 (an0) gp0 - entrada adc pino 7
   ansel=0b01100001;//fosc/64 / canal 0 (an0) gp0 - entrada adc pino 7

   cmcon= 0x07; //desliga comparadores analgicos
   gpio =0x00;//
   adcon0 = 0b10000001;// just dir/ canal 0 / liga conversor
   lcd_init();//inicializa o lcd
   printf(env_lcd_d,"  ZENER TESTER");//mostra nome do aparelho
   env_lcd_c(0xc1);
   printf(env_lcd_d," by PICSOURCE");//blog
   delay1seg();
   delay1seg();
   env_lcd_c(0x01);//limpar tela

   //



//==============================================================================
//         LAO DE REPETIO PRINCIPAL
//==============================================================================

   for(;;){

       leitura_adc();//leitura adc

     mil= res_esr/1000 | 0x30;
     res_esr= res_esr % 1000;

     cent=res_esr/100 |0x30;
     res_esr=res_esr%100;
     dez=res_esr/10 | 0x30;//converte valor encontrado para decimal dezenas
     un= res_esr%10 | 0x30;//converte valor encontrado para decimal unidades

      env_lcd_c(0x8b);

     env_lcd_c(0x81); //envia para linha 1 posio 1
     printf(env_lcd_d,"Tensao Zener:");




     env_lcd_c(0xc1); //envia para linha 2 posio 1


       if(mil == 0x30) env_lcd_d(' '); else env_lcd_d(mil);
       if(mil == 0x30 && cent == 0x30) env_lcd_d(' '); else env_lcd_d(cent);
       env_lcd_d(dez);
       env_lcd_d('.');
       env_lcd_d(un);

     printf(env_lcd_d," Volts");
     delay_ms(400);//tempo entre medies

 }//for(;;)

}//main()




